@@ -1,7 +1,7 @@
 /*
  * This file is part of Soprano Project.
  *
- * Copyright (C) 2007-2010 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,15 +20,12 @@
  */
 
 #include "servercore.h"
-#include "servercore_p.h"
 #include "soprano-server-config.h"
 #include "serverconnection.h"
 #ifdef BUILD_DBUS_SUPPORT
 #include "dbus/dbuscontroller.h"
 #endif
 #include "modelpool.h"
-#include "localserver.h"
-#include "tcpserver.h"
 
 #include "backend.h"
 #include "storagemodel.h"
@@ -40,31 +37,114 @@
 #include <QtCore/QDir>
 #include <QtNetwork/QTcpServer>
 #include <QtNetwork/QHostAddress>
+#include <QtNetwork/QLocalServer>
+#include <QtNetwork/QLocalSocket>
 #include <QtNetwork/QTcpSocket>
 #include <QtDBus/QtDBus>
 
 
+Q_DECLARE_METATYPE( QLocalSocket::LocalSocketError )
+Q_DECLARE_METATYPE( QAbstractSocket::SocketError )
+
+
 const quint16 Soprano::Server::ServerCore::DEFAULT_PORT = 5000;
 
 
-void Soprano::Server::ServerCorePrivate::addConnection( ServerConnection* conn )
+class Soprano::Server::ServerCore::Private
 {
-    connections.append( conn );
-    QObject::connect( conn, SIGNAL(finished()), q, SLOT(serverConnectionFinished()));
-    conn->start();
-    qDebug() << Q_FUNC_INFO << "New connection. New count:" << connections.count();
+public:
+    Private()
+        : maxConnectionCount( 0 ),
+#ifdef BUILD_DBUS_SUPPORT
+          dbusController( 0 ),
+#endif
+          tcpServer( 0 ),
+          socketServer( 0 )
+    {}
+
+    const Backend* backend;
+    BackendSettings settings;
+
+    int maxConnectionCount;
+
+    QHash<QString, Model*> models;
+    QList<ServerConnection*> connections;
+
+#ifdef BUILD_DBUS_SUPPORT
+    DBusController* dbusController;
+#endif
+
+    QTcpServer* tcpServer;
+    QLocalServer* socketServer;
+
+    // bridge between ServerCore and ServerConnection
+    ModelPool* modelPool;
+
+    ServerCore* q;
+
+    BackendSettings createBackendSettings( const QString& name ) {
+        BackendSettings newSettings = settings;
+        if ( isOptionInSettings( newSettings, BackendOptionStorageDir ) ) {
+            BackendSetting& setting = settingInSettings( newSettings, BackendOptionStorageDir );
+            setting.setValue( setting.value().toString() + '/' + name );
+        }
+        return newSettings;
+    }
+
+    void handleIncomingConnection( QIODevice* socket );
+
+    void _s_localSocketError( QLocalSocket::LocalSocketError );
+    void _s_tcpSocketError( QAbstractSocket::SocketError );
+};
+
+
+void Soprano::Server::ServerCore::Private::handleIncomingConnection( QIODevice* socket )
+{
+    qDebug() << Q_FUNC_INFO;
+    if ( maxConnectionCount > 0 &&
+         connections.count() >= maxConnectionCount ) {
+        qDebug() << Q_FUNC_INFO << "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA too many conenctions! go away!";
+        delete socket;
+    }
+    else {
+        ServerConnection* conn = new ServerConnection( modelPool, q );
+        connections.append( conn );
+        connect( conn, SIGNAL(finished()), q, SLOT(serverConnectionFinished()));
+        connect( socket, SIGNAL( error( QLocalSocket::LocalSocketError ) ),
+                 q, SLOT( _s_localSocketError( QLocalSocket::LocalSocketError ) ) );
+        conn->start( socket );
+        qDebug() << Q_FUNC_INFO << "New connection. New count:" << connections.count();
+    }
 }
 
 
+void Soprano::Server::ServerCore::Private::_s_localSocketError( QLocalSocket::LocalSocketError error )
+{
+    if( error != QLocalSocket::PeerClosedError ) {
+        qDebug() << "local socket error:" << error;
+    }
+}
+
+
+void Soprano::Server::ServerCore::Private::_s_tcpSocketError( QAbstractSocket::SocketError error )
+{
+    qDebug() << "tcp socket error:" << error;
+}
+
+
+
 Soprano::Server::ServerCore::ServerCore( QObject* parent )
     : QObject( parent ),
-      d( new ServerCorePrivate() )
+      d( new Private() )
 {
     d->q = this;
 
     // default backend
     d->backend = Soprano::usedBackend();
     d->modelPool = new ModelPool( this );
+
+    qRegisterMetaType<QLocalSocket::LocalSocketError>();
+    qRegisterMetaType<QAbstractSocket::SocketError>();
 }
 
 
@@ -126,9 +206,17 @@ Soprano::Model* Soprano::Server::ServerCore::model( const QString& name )
         }
 
         Model* model = createModel( settings );
-        d->models.insert( name, model );
 
-        return model;
+        // Although the server is now multithreaded we still have the DBus interface which is not
+        // It has its own dedicated thread but a deadlock can still be produced by a write operation
+        // which is issued while an iterator is open. By using the AsyncModel in SingleThreadMode
+        // we make sure that the DBus adaptor never deadlocks.
+
+        Util::AsyncModel* asyncModel = new Util::AsyncModel( model );
+        model->setParent( asyncModel ); // memory management
+        d->models.insert( name, asyncModel );
+
+        return asyncModel;
     }
     else {
         return *it;
@@ -165,7 +253,9 @@ bool Soprano::Server::ServerCore::listen( quint16 port )
 {
     clearError();
     if ( !d->tcpServer ) {
-        d->tcpServer = new TcpServer( d, this );
+        d->tcpServer = new QTcpServer( this );
+        connect( d->tcpServer, SIGNAL( newConnection() ),
+                 this, SLOT( slotNewTcpConnection() ) );
     }
 
     if ( !d->tcpServer->listen( QHostAddress::Any, port ) ) {
@@ -195,7 +285,9 @@ bool Soprano::Server::ServerCore::start( const QString& name )
 {
     clearError();
     if ( !d->socketServer ) {
-        d->socketServer = new LocalServer( d, this );
+        d->socketServer = new QLocalServer( this );
+        connect( d->socketServer, SIGNAL( newConnection() ),
+                 this, SLOT( slotNewSocketConnection() ) );
     }
 
     QString path( name );
@@ -260,4 +352,18 @@ QStringList Soprano::Server::ServerCore::allModels() const
     return d->models.keys();
 }
 
+
+void Soprano::Server::ServerCore::slotNewTcpConnection()
+{
+    qDebug() << Q_FUNC_INFO;
+    d->handleIncomingConnection( d->tcpServer->nextPendingConnection() );
+}
+
+
+void Soprano::Server::ServerCore::slotNewSocketConnection()
+{
+    qDebug() << Q_FUNC_INFO;
+    d->handleIncomingConnection( d->socketServer->nextPendingConnection() );
+}
+
 #include "servercore.moc"
